--[[
--
-- oUF_Rabbit
--
-- Custom layout for oUF.
-- Based on VuBce, rUnits, TsoHG and Lily.
-- by rabbit.magtheridon@gmail.com
--
--]]

local statusbarTexture = "Interface\\AddOns\\oUF_Rabbit\\media\\armory"
local borderTexture = "Interface\\AddOns\\oUF_Rabbit\\media\\border"
local font = "Interface\\AddOns\\oUF_Rabbit\\media\\font.ttf"
local media = AceLibrary:HasInstance("LibSharedMedia-3.0") and AceLibrary("LibSharedMedia-3.0") or nil
if media then
	media:Register("statusbar", "Armory", statusbarTexture)
	media:Register("border", "Roth", borderTexture)
	media:Register("font", "Accidental Presidency", font)
	-- Fetch again in case the user has set up some overrides
	-- XXX should register a callback for the overrides
	statusbarTexture = media:Fetch("statusbar", "Armory")
	borderTexture = media:Fetch("border", "Roth")
	font = media:Fetch("font", "Accidental Presidency")
end

local function menu(self)
	local unit = self.unit:sub(1, -2)
	local cunit = self.unit:gsub("^%l", string.upper)

	if unit == "party" or unit == "partypet" then
		ToggleDropDownMenu(1, nil, _G["PartyMemberFrame"..self.id.."DropDown"], "cursor", 0, 0)
	elseif _G[cunit.."FrameDropDown"] then
		ToggleDropDownMenu(1, nil, _G[cunit.."FrameDropDown"], "cursor", 0, 0)
	end
end

local function updateNameRaid(self, event, unit)
	if self.unit == unit or (not unit and self.unit) then
		local textFrame = self.Name
		local u = unit or self.unit
		if UnitIsDead(u) then
			textFrame:SetText("dead")
			textFrame:SetVertexColor(.5, .5, .5)
		elseif UnitIsGhost(u) then
			textFrame:SetText("ghost")
			textFrame:SetVertexColor(.5, .5, .5)
		elseif not UnitIsConnected(u) then
			textFrame:SetText("off")
			textFrame:SetVertexColor(.5, .5, .5)
		else
			local n = UnitName(u)
			textFrame:SetText(n:sub(1, 3))

			local _, c = UnitClass(u)
			local color = RAID_CLASS_COLORS[c]
			if color then
				textFrame:SetVertexColor(color.r, color.g, color.b)
			end
		end
	end
end

local function updateName(self, event, unit)
	if self.unit == unit or (not unit and self.unit) then
		local u = unit or self.unit
		self.Name:SetText(UnitName(u))

		if u ~= "targettarget" then
			local lvlFrame = self.Lvl
			local level = UnitLevel(u)
			local cl = UnitClassification(u)

			if not cl or cl == "normal" then
				lvlFrame:SetText(level)
			elseif cl == "worldboss" then
				lvlFrame:SetText("Boss")
			elseif cl == "rareelite" then
				lvlFrame:SetText(level .. "+ Rare")
			elseif cl == "elite" then
				lvlFrame:SetText(level .. "+")
			elseif cl == "rare" then
				lvlFrame:SetText(level .. " Rare")
			else
				lvlFrame:SetText(level)
			end
			lvlFrame:SetTextColor(1.0, 1.0, 1.0)

			local classFrame = self.Class
			if UnitIsPlayer(u) then
				local c, eC = UnitClass(u)
				classFrame:SetText(c)
				local color = RAID_CLASS_COLORS[eC]
				if color then
					classFrame:SetVertexColor(color.r, color.g, color.b)
				end
			else
				classFrame:SetVertexColor(1, 1, 1)
				classFrame:SetText(UnitCreatureFamily(u) or UnitCreatureType(u))
			end
			if u ~= "player" then
				self.Race:SetText(UnitRace(u))
			end
		end
	end
end

local function updateHealthRaid(self, event, bar, unit, min, max)
	if not UnitIsConnected(unit) or UnitIsGhost(unit) or UnitIsDead(unit) then
		local _, max = bar:GetMinMaxValues()
		bar:SetValue(max)
		bar:SetStatusBarColor(.6, .6, .6)
		bar.bg:SetVertexColor(.6, .6, .6, .25)
	else
		local _, c = UnitClass(unit)
		local color = RAID_CLASS_COLORS[c]
		if color then 
			bar:SetStatusBarColor(color.r, color.g, color.b)
			bar.bg:SetVertexColor(color.r, color.g, color.b, .25)
		end
	end

	self:UNIT_NAME_UPDATE(event, unit)
end

local function updateHealth(self, event, bar, unit, min, max)
	if UnitIsDead(unit) then
		bar:SetValue(0)
		bar.value:SetText("Dead")
	elseif UnitIsGhost(unit) then
		bar:SetValue(0)
		bar.value:SetText("Ghost")
	elseif not UnitIsConnected(unit) then
		bar:SetValue(0)		
		bar.value:SetText("Offline")
	else
		local c = max - min
		if c > 0 then
			bar.value:SetText(c)
		elseif max ~= min then
			bar.value:SetText(max)
		else
			bar.value:SetText("")
		end
	end

	if UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit) or not UnitIsConnected(unit) then
		bar:SetStatusBarColor(.5, .5, .3)
		if self.Power then
			self.Power:SetStatusBarColor(.6, .6, .6)
		end
	else
		local _, c = UnitClass(unit)
		local color = UnitIsFriend(unit, "player") and UnitIsPlayer(unit) and RAID_CLASS_COLORS[c] or UnitReactionColor[UnitReaction(unit, "player")]
		if color then
			bar:SetStatusBarColor(color.r, color.g, color.b)
			bar.bg:SetVertexColor(color.r, color.g, color.b, 0.25)
		end
		self:UNIT_NAME_UPDATE(event, unit)
	end
end

local power = oUF.colors.power
local function updatePower(self, event, bar, unit, min, max)
	if UnitIsDead(unit) or UnitIsGhost(unit) or not UnitIsConnected(unit) then
		bar:SetValue(0)
	end
	if UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit) or not UnitIsConnected(unit) then
		bar:SetStatusBarColor(.6, .6, .6)
	else
		local color = power[UnitPowerType(unit)]
		if color then
			bar:SetStatusBarColor(color.r, color.g, color.b)
			bar.bg:SetVertexColor(color.r, color.g, color.b)
		end
	end
end

local function auraIcon(self, button)
	local t = button:CreateTexture(nil, "OVERLAY")
	t:SetTexture(borderTexture)
	t:SetAllPoints(button)
	t:SetVertexColor(0.25, 0.25, 0.35)
end

local backdrop = {
	bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", 
	tile = true, tileSize = 16,
	insets = {left = -2, right = -2, top = -2, bottom = -2},
}

local function func(settings, self, unit)
	self.menu = menu
	self:EnableMouse(true)
	self:SetScript("OnEnter", UnitFrame_OnEnter)
	self:SetScript("OnLeave", UnitFrame_OnLeave)
	self:RegisterForClicks("anyup")
	self:SetAttribute("*type2", "menu")

	if not unit then -- Party
		self:SetWidth(168) 
		self:SetHeight(38)
	elseif unit == "targettarget" then
		self:SetWidth(96)
		self:SetHeight(20)
	elseif unit == "player" or unit == "target" then
		self:SetWidth(192)
		self:SetHeight(38)
	end

	self:SetBackdrop(backdrop)
	self:SetBackdropColor(0, 0, 0, .9)

	local hp = CreateFrame("StatusBar")
	hp:SetHeight(20)
	hp:SetStatusBarTexture(statusbarTexture)
	hp:SetStatusBarColor(0, 1, 0)

	hp:SetParent(self)
	hp:SetPoint("TOP")
	hp:SetPoint("LEFT")
	hp:SetPoint("RIGHT")

	local hpbg = hp:CreateTexture(nil, "BORDER")
	hpbg:SetAllPoints(hp)
	hpbg:SetTexture(statusbarTexture)
	hpbg:SetBlendMode("BLEND")
	hpbg:SetAlpha(0.25)

	local hpp = hp:CreateFontString(nil, "OVERLAY")
	hpp:SetPoint("RIGHT", -3, 0)
	hpp:SetFontObject(GameFontNormal)
	hpp:SetVertexColor(1, 1, 1)

	hp.bg = hpbg
	hp.value = hpp
	self.Health = hp
	self.OverrideUpdateHealth = updateHealth

	local name = hp:CreateFontString(nil, "OVERLAY")
	name:SetPoint("LEFT", 2, 0)
	name:SetPoint("RIGHT", hpp, "LEFT")
	name:SetJustifyH("LEFT")
	name:SetFont(font, 16)
	name:SetShadowColor(GameFontNormal:GetShadowColor())
	name:SetShadowOffset(GameFontNormal:GetShadowOffset())
	name:SetVertexColor(1, 1, 1)
	self.Name = name
	self.UNIT_NAME_UPDATE = updateName

	local ricon = hp:CreateTexture(nil, "OVERLAY")
	ricon:SetHeight(24)
	ricon:SetWidth(24)
	ricon:SetPoint("CENTER", hp, "TOP", 0, 0)
	ricon:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
	self.RaidIcon = ricon

	if unit ~= "targettarget" then
		local pp = CreateFrame("StatusBar")
		pp:SetHeight(16)
		pp:SetStatusBarTexture(statusbarTexture)
		pp:SetStatusBarColor(.25, .25, .35)

		pp:SetParent(self)
		pp:SetPoint("TOP", hp, "BOTTOM", 0, -1)
		pp:SetPoint("LEFT")
		pp:SetPoint("RIGHT")

		local ppbg = pp:CreateTexture(nil, "BORDER")
		ppbg:SetAllPoints(pp)
		ppbg:SetTexture(statusbarTexture)
		ppbg:SetBlendMode("BLEND")
		ppbg:SetAlpha(0.25)

		local ppp = pp:CreateFontString(nil, "OVERLAY")
		ppp:SetPoint("RIGHT", -3, 0)
		ppp:SetFontObject(GameFontHighlight)
		local fontName, _, fontFlags = ppp:GetFont()
		ppp:SetFont(fontName, 11, fontFlags)

		pp.value = ppp
		pp.bg = ppbg
		self.Power = pp
		self.OverrideUpdatePower = updatePower

		local lvl = pp:CreateFontString(nil, "OVERLAY")
		lvl:SetPoint("LEFT", pp, "LEFT", 2, 0)
		lvl:SetFontObject(GameFontHighlight)
		lvl:SetFont(fontName, 11, fontFlags)
		self.Lvl = lvl

		local class = pp:CreateFontString(nil, "OVERLAY")
		class:SetPoint("LEFT", lvl, "RIGHT",  2, 0)
		class:SetFontObject(GameFontHighlight)
		class:SetFont(fontName, 11, fontFlags)
		self.Class = class

		local race = pp:CreateFontString(nil, "OVERLAY")
		race:SetPoint("LEFT", class, "RIGHT",  1, 0)
		race:SetFontObject(GameFontHighlight)
		race:SetFont(fontName, 11, fontFlags)
		self.Race = race

		local leader = hp:CreateTexture(nil, "OVERLAY")
		leader:SetHeight(14)
		leader:SetWidth(14)
		leader:SetPoint("BOTTOMRIGHT", hp, "TOPLEFT", 7, -7)
		leader:SetTexture("Interface\\GroupFrame\\UI-Group-LeaderIcon")
		self.Leader = leader
	end

	local buffs, debuffs = nil, nil
	if not unit then
		debuffs = CreateFrame("Frame", nil, self)
		local s = self.Power:GetHeight() + self.Health:GetHeight()
		debuffs.size = s
		debuffs:SetHeight(s)
		debuffs:SetWidth(s * 3)
		debuffs:SetPoint("LEFT", self, "RIGHT", 4, 0)
		debuffs.initialAnchor = "TOPLEFT"
		debuffs.num = 3 -- max debuffs
		self.PostCreateAuraIcon = auraIcon
	elseif unit == "target" or unit == "player" then
		buffs = CreateFrame("Frame", nil, self)
		buffs.size = self:GetWidth() / 8
		buffs.num = 8
		buffs:SetHeight(buffs.size)
		buffs:SetWidth(self:GetWidth())

		buffs:SetPoint("BOTTOMLEFT", self, "TOPLEFT", 0, 4)
		buffs.initialAnchor = "BOTTOMLEFT"
		buffs["growth-x"] = "RIGHT"
		buffs["growth-y"] = "UP"
		buffs.filter = (unit == "player")

		debuffs = CreateFrame("Frame", nil, self)
		debuffs:SetPoint("BOTTOMLEFT", buffs, "TOPLEFT", 0, 2)
		debuffs:SetPoint("BOTTOMRIGHT", buffs, "TOPRIGHT", 0, 2)
		debuffs.size = self:GetWidth() / 8
		debuffs.num = 16
		debuffs:SetHeight(debuffs.size * 2)

		debuffs.initialAnchor = "BOTTOMRIGHT"
		debuffs["growth-x"] = "LEFT"
		debuffs["growth-y"] = "UP"
	end

	self.Buffs = buffs
	self.Debuffs = debuffs

	if unit == "target" then
		local cpoints = self:CreateFontString(nil, "OVERLAY")
		cpoints:SetPoint("RIGHT", self, "LEFT", -8, 1)
		cpoints:SetFont(DAMAGE_TEXT_FONT, 38, "OUTLINE")
		cpoints:SetVertexColor(1, 1, 1)
		cpoints:SetJustifyH("RIGHT")
		self.CPoints = cpoints
	end

	if not unit then
		self.Range = true
		self.inRangeAlpha = 1
		self.outsideRangeAlpha = .5
	end

	return self
end

oUF:RegisterStyle("Rabbit", setmetatable({}, {__call = func}))
oUF:SetActiveStyle("Rabbit")

local player = oUF:Spawn("player", "oUF_Player")
player:SetPoint("BOTTOM", -114, 160)
local target = oUF:Spawn("target", "oUF_Target")
target:SetPoint("BOTTOM", 114, 160)
local tot = oUF:Spawn("targettarget", "oUF_TargetTarget")
tot:SetPoint("TOPRIGHT", target, "BOTTOMRIGHT", 0, -8)
--tot:SetScale(0.95)

local party = oUF:Spawn("header", "oUF_Party")
party:SetPoint("TOPLEFT", 4, -40)
party:SetAttribute("yOffset", -8)
party:SetAttribute("showParty", true)
party:Show()

local toggleParty = CreateFrame("Frame")
toggleParty:SetScript("OnEvent", function()
	if GetNumRaidMembers() > 0 then
		party:Hide()
	elseif GetNumPartyMembers() > 0 then
		party:Show()
	end
end)
toggleParty:RegisterEvent("PARTY_MEMBERS_CHANGED")
toggleParty:RegisterEvent("PARTY_LEADER_CHANGED")
toggleParty:RegisterEvent("RAID_ROSTER_UPDATE")
toggleParty:RegisterEvent("PLAYER_LOGIN")

local function raid(settings, self, unit)
	self.menu = menu

	self:EnableMouse(true)
	self:SetScript("OnEnter", UnitFrame_OnEnter)
	self:SetScript("OnLeave", UnitFrame_OnLeave)
	self:RegisterForClicks("anyup")
	self:SetAttribute("*type2", "menu")

	self:SetWidth(38)
	self:SetHeight(24)

	self:SetBackdrop(backdrop)
	self:SetBackdropColor(0, 0, 0, 1)

	local hp = CreateFrame("StatusBar")
	hp:SetHeight(24)
	hp:SetStatusBarTexture(statusbarTexture)
	hp:SetOrientation("VERTICAL")
	hp:SetParent(self)
	hp:SetPoint("TOP")
	hp:SetPoint("LEFT")
	hp:SetPoint("RIGHT")

	local hpbg = hp:CreateTexture(nil, "BORDER")
	hpbg:SetAllPoints(hp)
	hpbg:SetTexture(statusbarTexture)

	hp.bg = hpbg
	self.Health = hp
	self.OverrideUpdateHealth = updateHealthRaid

	local name = hp:CreateFontString(nil, "OVERLAY")
	name:SetPoint("LEFT", hp, "LEFT", 0, 1)
	name:SetPoint("RIGHT", hp, "RIGHT", 0, 1)
	name:SetJustifyH("CENTER")
	name:SetFont(font, 13)
	name:SetShadowColor(GameFontNormal:GetShadowColor())
	name:SetShadowOffset(GameFontNormal:GetShadowOffset())
	self.Name = name
	self.UNIT_NAME_UPDATE = updateNameRaid

	local icon = hp:CreateTexture(nil, "OVERLAY")
	icon:SetHeight(16)
	icon:SetWidth(16)
	icon:SetPoint("TOP", self, 0, 8)
	icon:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
	self.RaidIcon = icon

	local leader = hp:CreateTexture(nil, "OVERLAY")
	leader:SetHeight(13)
	leader:SetWidth(13)
	leader:SetPoint("TOPLEFT", self, 3, 6)
	leader:SetTexture("Interface\\GroupFrame\\UI-Group-LeaderIcon")
	self.Leader = leader

	self.Range = true
	self.inRangeAlpha = 1
	self.outsideRangeAlpha = .6

	return self
end

oUF:RegisterStyle("Rabbit_Raid", setmetatable({}, {__call = raid}))
oUF:SetActiveStyle("Rabbit_Raid")

local raid = {}
for i = 1, 8 do
	local rGroup = oUF:Spawn("header", "oUF_Raid" .. i)
	rGroup:SetAttribute("groupFilter", tostring(i))
	rGroup:SetAttribute("showRaid", true)
	rGroup:SetAttribute("yOffset", -8)
	table.insert(raid, rGroup)
	if i == 1 then
		rGroup:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 4, -40)
	else
		rGroup:SetPoint("TOPLEFT", raid[i-1], "TOPRIGHT", 8, 0)		
	end
	rGroup:Show()
end

